from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.add_product_image_multipart_data import AddProductImageMultipartData
from ...models.response_image import ResponseImage
from ...types import Response


def _get_kwargs(
    product_code: str,
    *,
    client: Client,
    multipart_data: AddProductImageMultipartData,
) -> Dict[str, Any]:
    url = "{}/products/{productCode}/images".format(client.base_url, productCode=product_code)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    multipart_multipart_data = multipart_data.to_multipart()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "files": multipart_multipart_data,
    }


def _parse_response(*, response: httpx.Response) -> Optional[ResponseImage]:
    if response.status_code == 200:
        response_200 = ResponseImage.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[ResponseImage]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    product_code: str,
    *,
    client: Client,
    multipart_data: AddProductImageMultipartData,
) -> Response[ResponseImage]:
    """Add product image

     First, a product has to be created using```POST /products```, the response contain a product object
    upon a successful creation. Use the product code to for the add/delete images URLs.

    Use a standard file upload request (multipart form data) with a file attachment, parameter name is
    `file` as the call payload, and also a mandatory filename. If you have multiple images, you need to
    make one separate call for each image.

    A successful response contains generated image URLs including different image dimension and the
    image Id. Use the image Id to delete the image, if you want to remove it from the product.

    ### Request example

    File has to be specified either in the attachment, part of the form-data parameter 'file', or in the
    attachment, as a form-data parameter 'filename', which is common format for PHP frameworks, which
    are sending an array of attachments.

    Request:
    ```
    POST https://api.rezdy.com/latest/products/P12345/images?apiKey=123456789XYZ
    ```
    content-type=[multipart/form-data; boundary=----WebKitFormBoundarymDtt4W0lhmAsKFkZ]

    ```
    ------WebKitFormBoundarymDtt4W0lhmAsKFkZ
    Content-Disposition: form-data; name=\"file\"; filename=\"myImage.png\"
    Content-Type: image/png
    ...
    IMAGE BINARY DATA
    ...
    ------WebKitFormBoundarymDtt4W0lhmAsKFkZ--
    ```


    Args:
        product_code (str):
        multipart_data (AddProductImageMultipartData):

    Returns:
        Response[ResponseImage]
    """

    kwargs = _get_kwargs(
        product_code=product_code,
        client=client,
        multipart_data=multipart_data,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    product_code: str,
    *,
    client: Client,
    multipart_data: AddProductImageMultipartData,
) -> Optional[ResponseImage]:
    """Add product image

     First, a product has to be created using```POST /products```, the response contain a product object
    upon a successful creation. Use the product code to for the add/delete images URLs.

    Use a standard file upload request (multipart form data) with a file attachment, parameter name is
    `file` as the call payload, and also a mandatory filename. If you have multiple images, you need to
    make one separate call for each image.

    A successful response contains generated image URLs including different image dimension and the
    image Id. Use the image Id to delete the image, if you want to remove it from the product.

    ### Request example

    File has to be specified either in the attachment, part of the form-data parameter 'file', or in the
    attachment, as a form-data parameter 'filename', which is common format for PHP frameworks, which
    are sending an array of attachments.

    Request:
    ```
    POST https://api.rezdy.com/latest/products/P12345/images?apiKey=123456789XYZ
    ```
    content-type=[multipart/form-data; boundary=----WebKitFormBoundarymDtt4W0lhmAsKFkZ]

    ```
    ------WebKitFormBoundarymDtt4W0lhmAsKFkZ
    Content-Disposition: form-data; name=\"file\"; filename=\"myImage.png\"
    Content-Type: image/png
    ...
    IMAGE BINARY DATA
    ...
    ------WebKitFormBoundarymDtt4W0lhmAsKFkZ--
    ```


    Args:
        product_code (str):
        multipart_data (AddProductImageMultipartData):

    Returns:
        Response[ResponseImage]
    """

    return sync_detailed(
        product_code=product_code,
        client=client,
        multipart_data=multipart_data,
    ).parsed


async def asyncio_detailed(
    product_code: str,
    *,
    client: Client,
    multipart_data: AddProductImageMultipartData,
) -> Response[ResponseImage]:
    """Add product image

     First, a product has to be created using```POST /products```, the response contain a product object
    upon a successful creation. Use the product code to for the add/delete images URLs.

    Use a standard file upload request (multipart form data) with a file attachment, parameter name is
    `file` as the call payload, and also a mandatory filename. If you have multiple images, you need to
    make one separate call for each image.

    A successful response contains generated image URLs including different image dimension and the
    image Id. Use the image Id to delete the image, if you want to remove it from the product.

    ### Request example

    File has to be specified either in the attachment, part of the form-data parameter 'file', or in the
    attachment, as a form-data parameter 'filename', which is common format for PHP frameworks, which
    are sending an array of attachments.

    Request:
    ```
    POST https://api.rezdy.com/latest/products/P12345/images?apiKey=123456789XYZ
    ```
    content-type=[multipart/form-data; boundary=----WebKitFormBoundarymDtt4W0lhmAsKFkZ]

    ```
    ------WebKitFormBoundarymDtt4W0lhmAsKFkZ
    Content-Disposition: form-data; name=\"file\"; filename=\"myImage.png\"
    Content-Type: image/png
    ...
    IMAGE BINARY DATA
    ...
    ------WebKitFormBoundarymDtt4W0lhmAsKFkZ--
    ```


    Args:
        product_code (str):
        multipart_data (AddProductImageMultipartData):

    Returns:
        Response[ResponseImage]
    """

    kwargs = _get_kwargs(
        product_code=product_code,
        client=client,
        multipart_data=multipart_data,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    product_code: str,
    *,
    client: Client,
    multipart_data: AddProductImageMultipartData,
) -> Optional[ResponseImage]:
    """Add product image

     First, a product has to be created using```POST /products```, the response contain a product object
    upon a successful creation. Use the product code to for the add/delete images URLs.

    Use a standard file upload request (multipart form data) with a file attachment, parameter name is
    `file` as the call payload, and also a mandatory filename. If you have multiple images, you need to
    make one separate call for each image.

    A successful response contains generated image URLs including different image dimension and the
    image Id. Use the image Id to delete the image, if you want to remove it from the product.

    ### Request example

    File has to be specified either in the attachment, part of the form-data parameter 'file', or in the
    attachment, as a form-data parameter 'filename', which is common format for PHP frameworks, which
    are sending an array of attachments.

    Request:
    ```
    POST https://api.rezdy.com/latest/products/P12345/images?apiKey=123456789XYZ
    ```
    content-type=[multipart/form-data; boundary=----WebKitFormBoundarymDtt4W0lhmAsKFkZ]

    ```
    ------WebKitFormBoundarymDtt4W0lhmAsKFkZ
    Content-Disposition: form-data; name=\"file\"; filename=\"myImage.png\"
    Content-Type: image/png
    ...
    IMAGE BINARY DATA
    ...
    ------WebKitFormBoundarymDtt4W0lhmAsKFkZ--
    ```


    Args:
        product_code (str):
        multipart_data (AddProductImageMultipartData):

    Returns:
        Response[ResponseImage]
    """

    return (
        await asyncio_detailed(
            product_code=product_code,
            client=client,
            multipart_data=multipart_data,
        )
    ).parsed



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.response_no_data import ResponseNoData
from ...types import Response


def _get_kwargs(
    extra_id: int,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/extra/{extraId}".format(client.base_url, extraId=extra_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "delete",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[ResponseNoData]:
    if response.status_code == 200:
        response_200 = ResponseNoData.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[ResponseNoData]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    extra_id: int,
    *,
    client: Client,
) -> Response[ResponseNoData]:
    """Delete extra

     Deletes an extra

    Args:
        extra_id (int):

    Returns:
        Response[ResponseNoData]
    """

    kwargs = _get_kwargs(
        extra_id=extra_id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    extra_id: int,
    *,
    client: Client,
) -> Optional[ResponseNoData]:
    """Delete extra

     Deletes an extra

    Args:
        extra_id (int):

    Returns:
        Response[ResponseNoData]
    """

    return sync_detailed(
        extra_id=extra_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    extra_id: int,
    *,
    client: Client,
) -> Response[ResponseNoData]:
    """Delete extra

     Deletes an extra

    Args:
        extra_id (int):

    Returns:
        Response[ResponseNoData]
    """

    kwargs = _get_kwargs(
        extra_id=extra_id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    extra_id: int,
    *,
    client: Client,
) -> Optional[ResponseNoData]:
    """Delete extra

     Deletes an extra

    Args:
        extra_id (int):

    Returns:
        Response[ResponseNoData]
    """

    return (
        await asyncio_detailed(
            extra_id=extra_id,
            client=client,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.booking_field import BookingField
from ..types import UNSET, Unset

T = TypeVar("T", bound="Participant")


@attr.s(auto_attribs=True)
class Participant:
    """Details about a single participant for a single BookingItem.<br>The participant is a person attending a tour. It
    differs from the Customer, who is the person making the booking and most of the time paying for it.

        Attributes:
            fields (Union[Unset, List[BookingField]]): List of BookingField for this participant
    """

    fields: Union[Unset, List[BookingField]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        fields: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.fields, Unset):
            fields = []
            for fields_item_data in self.fields:
                fields_item = fields_item_data.to_dict()

                fields.append(fields_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if fields is not UNSET:
            field_dict["fields"] = fields

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        fields = []
        _fields = d.pop("fields", UNSET)
        for fields_item_data in _fields or []:
            fields_item = BookingField.from_dict(fields_item_data)

            fields.append(fields_item)

        participant = cls(
            fields=fields,
        )

        participant.additional_properties = d
        return participant

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.response_session_list import ResponseSessionList
from ...models.session_update_batch_request import SessionUpdateBatchRequest
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: SessionUpdateBatchRequest,
) -> Dict[str, Any]:
    url = "{}/availability/batch".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[ResponseSessionList]:
    if response.status_code == 200:
        response_200 = ResponseSessionList.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[ResponseSessionList]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: SessionUpdateBatchRequest,
) -> Response[ResponseSessionList]:
    """Update availability batch

     Batch Update availability for a specific product and time period.
    This will update availability for all the sessions that start in that time range. <br>
    Product code is optional, all sessions for all products will be updated if it is empty.
    You can use this service to blackout periods (I.e. set availability to 0 for a full day).

    Maximum range is one week.

    This can only update sessions from products with InventoryMode = SESSION_SEATS.
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        seatsAvailable: 0
    }
    ```
    OR
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        seats: 30
    }
    ```
    OR
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        priceOptions: [
           {
             price: 90,
             label: Adult
           }
         ]
    }
    ```

    If you send `seatsAvailable`, sessions will be updated for the current availability to become this
    number. <br>

    If you send `seats`, the total seats capacity of sessions will be updated, regardless of how many
    are already booked. <br>

    If you send both, only `seatsAvailable` will be used. <br>
    If you send priceOptions, they will override the default price options of the existing session.

    Args:
        json_body (SessionUpdateBatchRequest): Batch update session request data.

    Returns:
        Response[ResponseSessionList]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: SessionUpdateBatchRequest,
) -> Optional[ResponseSessionList]:
    """Update availability batch

     Batch Update availability for a specific product and time period.
    This will update availability for all the sessions that start in that time range. <br>
    Product code is optional, all sessions for all products will be updated if it is empty.
    You can use this service to blackout periods (I.e. set availability to 0 for a full day).

    Maximum range is one week.

    This can only update sessions from products with InventoryMode = SESSION_SEATS.
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        seatsAvailable: 0
    }
    ```
    OR
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        seats: 30
    }
    ```
    OR
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        priceOptions: [
           {
             price: 90,
             label: Adult
           }
         ]
    }
    ```

    If you send `seatsAvailable`, sessions will be updated for the current availability to become this
    number. <br>

    If you send `seats`, the total seats capacity of sessions will be updated, regardless of how many
    are already booked. <br>

    If you send both, only `seatsAvailable` will be used. <br>
    If you send priceOptions, they will override the default price options of the existing session.

    Args:
        json_body (SessionUpdateBatchRequest): Batch update session request data.

    Returns:
        Response[ResponseSessionList]
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: SessionUpdateBatchRequest,
) -> Response[ResponseSessionList]:
    """Update availability batch

     Batch Update availability for a specific product and time period.
    This will update availability for all the sessions that start in that time range. <br>
    Product code is optional, all sessions for all products will be updated if it is empty.
    You can use this service to blackout periods (I.e. set availability to 0 for a full day).

    Maximum range is one week.

    This can only update sessions from products with InventoryMode = SESSION_SEATS.
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        seatsAvailable: 0
    }
    ```
    OR
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        seats: 30
    }
    ```
    OR
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        priceOptions: [
           {
             price: 90,
             label: Adult
           }
         ]
    }
    ```

    If you send `seatsAvailable`, sessions will be updated for the current availability to become this
    number. <br>

    If you send `seats`, the total seats capacity of sessions will be updated, regardless of how many
    are already booked. <br>

    If you send both, only `seatsAvailable` will be used. <br>
    If you send priceOptions, they will override the default price options of the existing session.

    Args:
        json_body (SessionUpdateBatchRequest): Batch update session request data.

    Returns:
        Response[ResponseSessionList]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: SessionUpdateBatchRequest,
) -> Optional[ResponseSessionList]:
    """Update availability batch

     Batch Update availability for a specific product and time period.
    This will update availability for all the sessions that start in that time range. <br>
    Product code is optional, all sessions for all products will be updated if it is empty.
    You can use this service to blackout periods (I.e. set availability to 0 for a full day).

    Maximum range is one week.

    This can only update sessions from products with InventoryMode = SESSION_SEATS.
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        seatsAvailable: 0
    }
    ```
    OR
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        seats: 30
    }
    ```
    OR
    ```
    {
        startTime: 2014-11-11T00:00:00Z,
        endTime: 2014-11-11T23:59:59Z,
        productCode: P123456,
        priceOptions: [
           {
             price: 90,
             label: Adult
           }
         ]
    }
    ```

    If you send `seatsAvailable`, sessions will be updated for the current availability to become this
    number. <br>

    If you send `seats`, the total seats capacity of sessions will be updated, regardless of how many
    are already booked. <br>

    If you send both, only `seatsAvailable` will be used. <br>
    If you send priceOptions, they will override the default price options of the existing session.

    Args:
        json_body (SessionUpdateBatchRequest): Batch update session request data.

    Returns:
        Response[ResponseSessionList]
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed



from enum import Enum


class BookingPaymentRecipient(str, Enum):
    SUPPLIER = "SUPPLIER"
    RESELLER = "RESELLER"
    REZDY = "REZDY"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class BookingUpdateStatus(str, Enum):
    PROCESSING = "PROCESSING"
    NEW = "NEW"
    ON_HOLD = "ON_HOLD"
    PENDING_SUPPLIER = "PENDING_SUPPLIER"
    PENDING_CUSTOMER = "PENDING_CUSTOMER"
    CONFIRMED = "CONFIRMED"
    CANCELLED = "CANCELLED"
    ABANDONED_CART = "ABANDONED_CART"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.booking_payment_currency import BookingPaymentCurrency
from ..models.booking_payment_recipient import BookingPaymentRecipient
from ..models.booking_payment_type import BookingPaymentType
from ..types import UNSET, Unset

T = TypeVar("T", bound="BookingPayment")


@attr.s(auto_attribs=True)
class BookingPayment:
    """Record of an already processed payment.

    Attributes:
        amount (Union[Unset, float]): Payment amount
        currency (Union[Unset, BookingPaymentCurrency]): Currency for this payment<br>Payments must be in the same
            currency than the order's totalCurrency.
        date (Union[Unset, datetime.datetime]): Date this payment was processed
        label (Union[Unset, str]): Reference or transaction code
        recipient (Union[Unset, BookingPaymentRecipient]): Payment recipient.
        type (Union[Unset, BookingPaymentType]): Type of payment
    """

    amount: Union[Unset, float] = UNSET
    currency: Union[Unset, BookingPaymentCurrency] = UNSET
    date: Union[Unset, datetime.datetime] = UNSET
    label: Union[Unset, str] = UNSET
    recipient: Union[Unset, BookingPaymentRecipient] = UNSET
    type: Union[Unset, BookingPaymentType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        amount = self.amount
        currency: Union[Unset, str] = UNSET
        if not isinstance(self.currency, Unset):
            currency = self.currency.value

        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        label = self.label
        recipient: Union[Unset, str] = UNSET
        if not isinstance(self.recipient, Unset):
            recipient = self.recipient.value

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if amount is not UNSET:
            field_dict["amount"] = amount
        if currency is not UNSET:
            field_dict["currency"] = currency
        if date is not UNSET:
            field_dict["date"] = date
        if label is not UNSET:
            field_dict["label"] = label
        if recipient is not UNSET:
            field_dict["recipient"] = recipient
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        amount = d.pop("amount", UNSET)

        _currency = d.pop("currency", UNSET)
        currency: Union[Unset, BookingPaymentCurrency]
        if isinstance(_currency, Unset):
            currency = UNSET
        else:
            currency = BookingPaymentCurrency(_currency)

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.datetime]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date)

        label = d.pop("label", UNSET)

        _recipient = d.pop("recipient", UNSET)
        recipient: Union[Unset, BookingPaymentRecipient]
        if isinstance(_recipient, Unset):
            recipient = UNSET
        else:
            recipient = BookingPaymentRecipient(_recipient)

        _type = d.pop("type", UNSET)
        type: Union[Unset, BookingPaymentType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = BookingPaymentType(_type)

        booking_payment = cls(
            amount=amount,
            currency=currency,
            date=date,
            label=label,
            recipient=recipient,
            type=type,
        )

        booking_payment.additional_properties = d
        return booking_payment

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.address import Address
from ..models.booking_field_create import BookingFieldCreate
from ..models.extra import Extra
from ..models.price_option_create import PriceOptionCreate
from ..models.product_create_request_barcode_output_type import ProductCreateRequestBarcodeOutputType
from ..models.product_create_request_booking_mode import ProductCreateRequestBookingMode
from ..models.product_create_request_confirm_mode import ProductCreateRequestConfirmMode
from ..models.product_create_request_product_type import ProductCreateRequestProductType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProductCreateRequest")


@attr.s(auto_attribs=True)
class ProductCreateRequest:
    """Partial product model containing all fields which are currently supported in product create via API.

    Attributes:
        booking_fields (List[BookingFieldCreate]): List of booking fields required for this product.
        booking_mode (ProductCreateRequestBookingMode): Booking mode. Determines if this product needs availability or
            can be booked for any date.
        confirm_mode (ProductCreateRequestConfirmMode): Confirmation mode. Determines if bookings are automatically
            confirmed or it they are pending.
        description (str): Long product description, is between 100 and 15000 characters.
        duration_minutes (int): Duration of the product in minutes.
        name (str): Product name
        price_options (List[PriceOptionCreate]): List of price options belonging to this product.
        product_type (ProductCreateRequestProductType): Type of this product.
        short_description (str): Product description is between 15 and 240 characters.
        unit_label (str): What a quantity for this product is. It can be people (I.e. participant, passenger, diver) or
            objects (Kayak, Helicopter, etc.)
        unit_label_plural (str): Plural version of unitLabel.
        additional_information (Union[Unset, str]): Additional information for the product, that should be sent after a
            booking is completed (i.e. by email) to the customer. Useful for integration, when manual control of the entire
            customer booking experience is wanted, and the automatic confirmation e-mail from Rezdy had been suppressed.
        advertised_price (Union[Unset, float]): General price indication for this product. It represents a display price
            only, therefore it does not affect a real booking price, which is calculated based on the price options.
        barcode_output_type (Union[Unset, ProductCreateRequestBarcodeOutputType]): Specifies how to output the barcodes
            when this product is booked. Valid types are:<br><li>PARTICIPANT: Barcodes will be generated by rezdy for each
            participant when an booking is created for this product</li><li>ORDER: Barcodes will be generated by rezdy per
            booking</li>
        charter (Union[Unset, bool]): A charter product means each session can only have a single booking, whatever the
            number of seats booked.
        confirm_mode_min_participants (Union[Unset, int]): If confirmMode is MANUAL_THEN_AUTO or AUTO_THEN_MANUAL,
            determines the minimum number of participants per booking to trigger the change.
        extras (Union[Unset, List[Extra]]): List of extras IDs.
        internal_code (Union[Unset, str]): Supplier-defined product code, used internally by the supplier.
        languages (Union[Unset, List[str]]): List of product languages. The format of the language is ISO 639 two-letter
            code with BCP 47 language variants, separated by underscore e.g. en_au.
        location_address (Union[Unset, Address]): Address of a company, customer or product location.
        minimum_notice_minutes (Union[Unset, int]): Minimum book ahead interval for he product in minutes.
        pickup_id (Union[Unset, int]): If pickups are configured for this product, the field will contain the id of the
            pickup location list created by the supplier.
        quantity_required (Union[Unset, bool]): Does this product require a quantity to be booked? True for most
            products. Can be false if the supplier can only provide one quantity at any single time (I.e. private charters)
            or a price of a booking is fixed regardless of quantity
        quantity_required_max (Union[Unset, int]): Represent the max booking quantity for the product. It can be setup
            for a supplier product. For a successful booking of the product, the total number of participants (regardless of
            pricing options), per booking item in the booking request, have to be lesser or equal than this value.
        quantity_required_min (Union[Unset, int]): Represent the min booking quantity for the product. It can be setup
            for a supplier product. For a successful booking of the product, the total number of participants (regardless of
            pricing options), per booking item in the booking request, have to be greater or equal than this value.
        terms (Union[Unset, str]): Specific terms and conditions for this product.
        xero_account (Union[Unset, str]): Supplier Xero account for this product.
    """

    booking_fields: List[BookingFieldCreate]
    booking_mode: ProductCreateRequestBookingMode
    confirm_mode: ProductCreateRequestConfirmMode
    description: str
    duration_minutes: int
    name: str
    price_options: List[PriceOptionCreate]
    product_type: ProductCreateRequestProductType
    short_description: str
    unit_label: str
    unit_label_plural: str
    additional_information: Union[Unset, str] = UNSET
    advertised_price: Union[Unset, float] = UNSET
    barcode_output_type: Union[Unset, ProductCreateRequestBarcodeOutputType] = UNSET
    charter: Union[Unset, bool] = UNSET
    confirm_mode_min_participants: Union[Unset, int] = UNSET
    extras: Union[Unset, List[Extra]] = UNSET
    internal_code: Union[Unset, str] = UNSET
    languages: Union[Unset, List[str]] = UNSET
    location_address: Union[Unset, Address] = UNSET
    minimum_notice_minutes: Union[Unset, int] = UNSET
    pickup_id: Union[Unset, int] = UNSET
    quantity_required: Union[Unset, bool] = UNSET
    quantity_required_max: Union[Unset, int] = UNSET
    quantity_required_min: Union[Unset, int] = UNSET
    terms: Union[Unset, str] = UNSET
    xero_account: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        booking_fields = []
        for booking_fields_item_data in self.booking_fields:
            booking_fields_item = booking_fields_item_data.to_dict()

            booking_fields.append(booking_fields_item)

        booking_mode = self.booking_mode.value

        confirm_mode = self.confirm_mode.value

        description = self.description
        duration_minutes = self.duration_minutes
        name = self.name
        price_options = []
        for price_options_item_data in self.price_options:
            price_options_item = price_options_item_data.to_dict()

            price_options.append(price_options_item)

        product_type = self.product_type.value

        short_description = self.short_description
        unit_label = self.unit_label
        unit_label_plural = self.unit_label_plural
        additional_information = self.additional_information
        advertised_price = self.advertised_price
        barcode_output_type: Union[Unset, str] = UNSET
        if not isinstance(self.barcode_output_type, Unset):
            barcode_output_type = self.barcode_output_type.value

        charter = self.charter
        confirm_mode_min_participants = self.confirm_mode_min_participants
        extras: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.extras, Unset):
            extras = []
            for extras_item_data in self.extras:
                extras_item = extras_item_data.to_dict()

                extras.append(extras_item)

        internal_code = self.internal_code
        languages: Union[Unset, List[str]] = UNSET
        if not isinstance(self.languages, Unset):
            languages = self.languages

        location_address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.location_address, Unset):
            location_address = self.location_address.to_dict()

        minimum_notice_minutes = self.minimum_notice_minutes
        pickup_id = self.pickup_id
        quantity_required = self.quantity_required
        quantity_required_max = self.quantity_required_max
        quantity_required_min = self.quantity_required_min
        terms = self.terms
        xero_account = self.xero_account

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "bookingFields": booking_fields,
                "bookingMode": booking_mode,
                "confirmMode": confirm_mode,
                "description": description,
                "durationMinutes": duration_minutes,
                "name": name,
                "priceOptions": price_options,
                "productType": product_type,
                "shortDescription": short_description,
                "unitLabel": unit_label,
                "unitLabelPlural": unit_label_plural,
            }
        )
        if additional_information is not UNSET:
            field_dict["additionalInformation"] = additional_information
        if advertised_price is not UNSET:
            field_dict["advertisedPrice"] = advertised_price
        if barcode_output_type is not UNSET:
            field_dict["barcodeOutputType"] = barcode_output_type
        if charter is not UNSET:
            field_dict["charter"] = charter
        if confirm_mode_min_participants is not UNSET:
            field_dict["confirmModeMinParticipants"] = confirm_mode_min_participants
        if extras is not UNSET:
            field_dict["extras"] = extras
        if internal_code is not UNSET:
            field_dict["internalCode"] = internal_code
        if languages is not UNSET:
            field_dict["languages"] = languages
        if location_address is not UNSET:
            field_dict["locationAddress"] = location_address
        if minimum_notice_minutes is not UNSET:
            field_dict["minimumNoticeMinutes"] = minimum_notice_minutes
        if pickup_id is not UNSET:
            field_dict["pickupId"] = pickup_id
        if quantity_required is not UNSET:
            field_dict["quantityRequired"] = quantity_required
        if quantity_required_max is not UNSET:
            field_dict["quantityRequiredMax"] = quantity_required_max
        if quantity_required_min is not UNSET:
            field_dict["quantityRequiredMin"] = quantity_required_min
        if terms is not UNSET:
            field_dict["terms"] = terms
        if xero_account is not UNSET:
            field_dict["xeroAccount"] = xero_account

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        booking_fields = []
        _booking_fields = d.pop("bookingFields")
        for booking_fields_item_data in _booking_fields:
            booking_fields_item = BookingFieldCreate.from_dict(booking_fields_item_data)

            booking_fields.append(booking_fields_item)

        booking_mode = ProductCreateRequestBookingMode(d.pop("bookingMode"))

        confirm_mode = ProductCreateRequestConfirmMode(d.pop("confirmMode"))

        description = d.pop("description")

        duration_minutes = d.pop("durationMinutes")

        name = d.pop("name")

        price_options = []
        _price_options = d.pop("priceOptions")
        for price_options_item_data in _price_options:
            price_options_item = PriceOptionCreate.from_dict(price_options_item_data)

            price_options.append(price_options_item)

        product_type = ProductCreateRequestProductType(d.pop("productType"))

        short_description = d.pop("shortDescription")

        unit_label = d.pop("unitLabel")

        unit_label_plural = d.pop("unitLabelPlural")

        additional_information = d.pop("additionalInformation", UNSET)

        advertised_price = d.pop("advertisedPrice", UNSET)

        _barcode_output_type = d.pop("barcodeOutputType", UNSET)
        barcode_output_type: Union[Unset, ProductCreateRequestBarcodeOutputType]
        if isinstance(_barcode_output_type, Unset):
            barcode_output_type = UNSET
        else:
            barcode_output_type = ProductCreateRequestBarcodeOutputType(_barcode_output_type)

        charter = d.pop("charter", UNSET)

        confirm_mode_min_participants = d.pop("confirmModeMinParticipants", UNSET)

        extras = []
        _extras = d.pop("extras", UNSET)
        for extras_item_data in _extras or []:
            extras_item = Extra.from_dict(extras_item_data)

            extras.append(extras_item)

        internal_code = d.pop("internalCode", UNSET)

        languages = cast(List[str], d.pop("languages", UNSET))

        _location_address = d.pop("locationAddress", UNSET)
        location_address: Union[Unset, Address]
        if isinstance(_location_address, Unset):
            location_address = UNSET
        else:
            location_address = Address.from_dict(_location_address)

        minimum_notice_minutes = d.pop("minimumNoticeMinutes", UNSET)

        pickup_id = d.pop("pickupId", UNSET)

        quantity_required = d.pop("quantityRequired", UNSET)

        quantity_required_max = d.pop("quantityRequiredMax", UNSET)

        quantity_required_min = d.pop("quantityRequiredMin", UNSET)

        terms = d.pop("terms", UNSET)

        xero_account = d.pop("xeroAccount", UNSET)

        product_create_request = cls(
            booking_fields=booking_fields,
            booking_mode=booking_mode,
            confirm_mode=confirm_mode,
            description=description,
            duration_minutes=duration_minutes,
            name=name,
            price_options=price_options,
            product_type=product_type,
            short_description=short_description,
            unit_label=unit_label,
            unit_label_plural=unit_label_plural,
            additional_information=additional_information,
            advertised_price=advertised_price,
            barcode_output_type=barcode_output_type,
            charter=charter,
            confirm_mode_min_participants=confirm_mode_min_participants,
            extras=extras,
            internal_code=internal_code,
            languages=languages,
            location_address=location_address,
            minimum_notice_minutes=minimum_notice_minutes,
            pickup_id=pickup_id,
            quantity_required=quantity_required,
            quantity_required_max=quantity_required_max,
            quantity_required_min=quantity_required_min,
            terms=terms,
            xero_account=xero_account,
        )

        product_create_request.additional_properties = d
        return product_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.request_status import RequestStatus
from ..models.rezdy_connect_settings import RezdyConnectSettings
from ..types import UNSET, Unset

T = TypeVar("T", bound="ResponseRezdyConnectSettings")


@attr.s(auto_attribs=True)
class ResponseRezdyConnectSettings:
    """
    Attributes:
        request_status (RequestStatus):
        rezdy_connect_settings (Union[Unset, RezdyConnectSettings]): Extended product model to support advanced product
            configuration, including RezdyConnect settings
    """

    request_status: RequestStatus
    rezdy_connect_settings: Union[Unset, RezdyConnectSettings] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_status = self.request_status.to_dict()

        rezdy_connect_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.rezdy_connect_settings, Unset):
            rezdy_connect_settings = self.rezdy_connect_settings.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestStatus": request_status,
            }
        )
        if rezdy_connect_settings is not UNSET:
            field_dict["rezdyConnectSettings"] = rezdy_connect_settings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        request_status = RequestStatus.from_dict(d.pop("requestStatus"))

        _rezdy_connect_settings = d.pop("rezdyConnectSettings", UNSET)
        rezdy_connect_settings: Union[Unset, RezdyConnectSettings]
        if isinstance(_rezdy_connect_settings, Unset):
            rezdy_connect_settings = UNSET
        else:
            rezdy_connect_settings = RezdyConnectSettings.from_dict(_rezdy_connect_settings)

        response_rezdy_connect_settings = cls(
            request_status=request_status,
            rezdy_connect_settings=rezdy_connect_settings,
        )

        response_rezdy_connect_settings.additional_properties = d
        return response_rezdy_connect_settings

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Image")


@attr.s(auto_attribs=True)
class Image:
    """Image links.

    Attributes:
        id (Union[Unset, int]): Id of the image. Can be used for DELETE /{productCode}/image/{mediaId} service
        item_url (Union[Unset, str]): Full size image link
        large_size_url (Union[Unset, str]): Large size image link (1280px)
        medium_size_url (Union[Unset, str]): Medium size image link (480px)
        thumbnail_url (Union[Unset, str]): Thumbnail image link (240px)
    """

    id: Union[Unset, int] = UNSET
    item_url: Union[Unset, str] = UNSET
    large_size_url: Union[Unset, str] = UNSET
    medium_size_url: Union[Unset, str] = UNSET
    thumbnail_url: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        item_url = self.item_url
        large_size_url = self.large_size_url
        medium_size_url = self.medium_size_url
        thumbnail_url = self.thumbnail_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if item_url is not UNSET:
            field_dict["itemUrl"] = item_url
        if large_size_url is not UNSET:
            field_dict["largeSizeUrl"] = large_size_url
        if medium_size_url is not UNSET:
            field_dict["mediumSizeUrl"] = medium_size_url
        if thumbnail_url is not UNSET:
            field_dict["thumbnailUrl"] = thumbnail_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        item_url = d.pop("itemUrl", UNSET)

        large_size_url = d.pop("largeSizeUrl", UNSET)

        medium_size_url = d.pop("mediumSizeUrl", UNSET)

        thumbnail_url = d.pop("thumbnailUrl", UNSET)

        image = cls(
            id=id,
            item_url=item_url,
            large_size_url=large_size_url,
            medium_size_url=medium_size_url,
            thumbnail_url=thumbnail_url,
        )

        image.additional_properties = d
        return image

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class BookingPaymentType(str, Enum):
    PAYPAL = "PAYPAL"
    CASH = "CASH"
    CREDITCARD = "CREDITCARD"
    BANKTRANSFER = "BANKTRANSFER"
    BANKCHEQUE = "BANKCHEQUE"
    REFUND = "REFUND"
    VOUCHER = "VOUCHER"
    PROMO_CODE = "PROMO_CODE"
    FREE = "FREE"
    OTHER = "OTHER"
    INVOICE = "INVOICE"
    REZDY_PAYOUTS = "REZDY_PAYOUTS"
    ALIPAY = "ALIPAY"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.product_update_request import ProductUpdateRequest
from ...models.response_product import ResponseProduct
from ...types import Response


def _get_kwargs(
    product_code: str,
    *,
    client: Client,
    json_body: ProductUpdateRequest,
) -> Dict[str, Any]:
    url = "{}/products/{productCode}".format(client.base_url, productCode=product_code)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[ResponseProduct]:
    if response.status_code == 200:
        response_200 = ResponseProduct.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[ResponseProduct]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    product_code: str,
    *,
    client: Client,
    json_body: ProductUpdateRequest,
) -> Response[ResponseProduct]:
    """Update product

     Updates a product.

    When updating price options, the full list of existing price options must be supplied in the update.
    Otherwise the system will remove any missing price options in the request from the product.

    For instance, if a product has 2 price options Adult and Child but the update request only contains
    Adult, the Child price option will be removed from the product.<br>
    Adding a price option works the same way. If the update request contains an extra price option, it
    will be added to the product.

    When price option values are updated via API, this will override all existing price in availability
    (session) to reflect the product price.
    If a different price in calendar/session is required to the product price, please make the changes
    to the product directly in your Rezdy account and select `do not change session price` in the page
    that follows after saving your changes.,


    Args:
        product_code (str):
        json_body (ProductUpdateRequest): Partial product model containing all fields which are
            currently supported in product create via API.

    Returns:
        Response[ResponseProduct]
    """

    kwargs = _get_kwargs(
        product_code=product_code,
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    product_code: str,
    *,
    client: Client,
    json_body: ProductUpdateRequest,
) -> Optional[ResponseProduct]:
    """Update product

     Updates a product.

    When updating price options, the full list of existing price options must be supplied in the update.
    Otherwise the system will remove any missing price options in the request from the product.

    For instance, if a product has 2 price options Adult and Child but the update request only contains
    Adult, the Child price option will be removed from the product.<br>
    Adding a price option works the same way. If the update request contains an extra price option, it
    will be added to the product.

    When price option values are updated via API, this will override all existing price in availability
    (session) to reflect the product price.
    If a different price in calendar/session is required to the product price, please make the changes
    to the product directly in your Rezdy account and select `do not change session price` in the page
    that follows after saving your changes.,


    Args:
        product_code (str):
        json_body (ProductUpdateRequest): Partial product model containing all fields which are
            currently supported in product create via API.

    Returns:
        Response[ResponseProduct]
    """

    return sync_detailed(
        product_code=product_code,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    product_code: str,
    *,
    client: Client,
    json_body: ProductUpdateRequest,
) -> Response[ResponseProduct]:
    """Update product

     Updates a product.

    When updating price options, the full list of existing price options must be supplied in the update.
    Otherwise the system will remove any missing price options in the request from the product.

    For instance, if a product has 2 price options Adult and Child but the update request only contains
    Adult, the Child price option will be removed from the product.<br>
    Adding a price option works the same way. If the update request contains an extra price option, it
    will be added to the product.

    When price option values are updated via API, this will override all existing price in availability
    (session) to reflect the product price.
    If a different price in calendar/session is required to the product price, please make the changes
    to the product directly in your Rezdy account and select `do not change session price` in the page
    that follows after saving your changes.,


    Args:
        product_code (str):
        json_body (ProductUpdateRequest): Partial product model containing all fields which are
            currently supported in product create via API.

    Returns:
        Response[ResponseProduct]
    """

    kwargs = _get_kwargs(
        product_code=product_code,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    product_code: str,
    *,
    client: Client,
    json_body: ProductUpdateRequest,
) -> Optional[ResponseProduct]:
    """Update product

     Updates a product.

    When updating price options, the full list of existing price options must be supplied in the update.
    Otherwise the system will remove any missing price options in the request from the product.

    For instance, if a product has 2 price options Adult and Child but the update request only contains
    Adult, the Child price option will be removed from the product.<br>
    Adding a price option works the same way. If the update request contains an extra price option, it
    will be added to the product.

    When price option values are updated via API, this will override all existing price in availability
    (session) to reflect the product price.
    If a different price in calendar/session is required to the product price, please make the changes
    to the product directly in your Rezdy account and select `do not change session price` in the page
    that follows after saving your changes.,


    Args:
        product_code (str):
        json_body (ProductUpdateRequest): Partial product model containing all fields which are
            currently supported in product create via API.

    Returns:
        Response[ResponseProduct]
    """

    return (
        await asyncio_detailed(
            product_code=product_code,
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PickupLocation")


@attr.s(auto_attribs=True)
class PickupLocation:
    """PickupLocation object. Holds information about the a pickup location from the pickup list configured for the
    product.

        Attributes:
            additional_instructions (Union[Unset, str]): Additional instructions for the pickup location.
            address (Union[Unset, str]): Address of the pickup location<br>In a booking item object, it represents a chosen
                pickup address for the booked item.
            latitude (Union[Unset, float]): google maps calculated latitude of the pickup address
            location_name (Union[Unset, str]): <p>Pickup location name - free text name for the location.</p>In a booking
                item object, it represents customer's pickup location name (if configured on product). It can be one name from
                pickup locations list of the booked product, or free text in case of the other pickup location option.<p>The
                value will be ignored, if the product does not allow pickups or if the location name does not match one of the
                product's pickup locations and 'other' pickup option is not enabled for the product pickup.</p>
            longitude (Union[Unset, float]): google maps calculated latitude of the pickup address
            minutes_prior (Union[Unset, int]): Pickup time in minutes, prior to the tour start time.
            pickup_instructions (Union[Unset, str]): <p>Present only in booking service response</p>Chosen pickup
                instructions (general and location specific). Shown when the pickup was chosen for the booked item.
            pickup_time (Union[Unset, str]): <p>Present only in booking service response</p>In a booking item object, it
                represents a calculated pickup time, in supplier's local timezone. Shown when the pickup was chosen for the
                booked item and pickup location contains duration.
    """

    additional_instructions: Union[Unset, str] = UNSET
    address: Union[Unset, str] = UNSET
    latitude: Union[Unset, float] = UNSET
    location_name: Union[Unset, str] = UNSET
    longitude: Union[Unset, float] = UNSET
    minutes_prior: Union[Unset, int] = UNSET
    pickup_instructions: Union[Unset, str] = UNSET
    pickup_time: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        additional_instructions = self.additional_instructions
        address = self.address
        latitude = self.latitude
        location_name = self.location_name
        longitude = self.longitude
        minutes_prior = self.minutes_prior
        pickup_instructions = self.pickup_instructions
        pickup_time = self.pickup_time

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if additional_instructions is not UNSET:
            field_dict["additionalInstructions"] = additional_instructions
        if address is not UNSET:
            field_dict["address"] = address
        if latitude is not UNSET:
            field_dict["latitude"] = latitude
        if location_name is not UNSET:
            field_dict["locationName"] = location_name
        if longitude is not UNSET:
            field_dict["longitude"] = longitude
        if minutes_prior is not UNSET:
            field_dict["minutesPrior"] = minutes_prior
        if pickup_instructions is not UNSET:
            field_dict["pickupInstructions"] = pickup_instructions
        if pickup_time is not UNSET:
            field_dict["pickupTime"] = pickup_time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        additional_instructions = d.pop("additionalInstructions", UNSET)

        address = d.pop("address", UNSET)

        latitude = d.pop("latitude", UNSET)

        location_name = d.pop("locationName", UNSET)

        longitude = d.pop("longitude", UNSET)

        minutes_prior = d.pop("minutesPrior", UNSET)

        pickup_instructions = d.pop("pickupInstructions", UNSET)

        pickup_time = d.pop("pickupTime", UNSET)

        pickup_location = cls(
            additional_instructions=additional_instructions,
            address=address,
            latitude=latitude,
            location_name=location_name,
            longitude=longitude,
            minutes_prior=minutes_prior,
            pickup_instructions=pickup_instructions,
            pickup_time=pickup_time,
        )

        pickup_location.additional_properties = d
        return pickup_location

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class VoucherStatus(str, Enum):
    ISSUED = "ISSUED"
    REDEEMED = "REDEEMED"
    PARTIALLY_REDEEMED = "PARTIALLY_REDEEMED"
    EXPIRED = "EXPIRED"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.category_seo_tag import CategorySeoTag
from ..models.image import Image
from ..types import UNSET, Unset

T = TypeVar("T", bound="Category")


@attr.s(auto_attribs=True)
class Category:
    """A Category is used to group products

    Attributes:
        category_seo_tags (Union[Unset, List[CategorySeoTag]]): This will store category meta data such as description
        description (Union[Unset, str]): Category description
        id (Union[Unset, int]): Category ID
        image (Union[Unset, Image]): Image links.
        name (Union[Unset, str]): Category name
        visible (Union[Unset, bool]): Flag used to determine if the category is public or private.<br>Public categories
            appear as tabs on the company booking form
    """

    category_seo_tags: Union[Unset, List[CategorySeoTag]] = UNSET
    description: Union[Unset, str] = UNSET
    id: Union[Unset, int] = UNSET
    image: Union[Unset, Image] = UNSET
    name: Union[Unset, str] = UNSET
    visible: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        category_seo_tags: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.category_seo_tags, Unset):
            category_seo_tags = []
            for category_seo_tags_item_data in self.category_seo_tags:
                category_seo_tags_item = category_seo_tags_item_data.to_dict()

                category_seo_tags.append(category_seo_tags_item)

        description = self.description
        id = self.id
        image: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.image, Unset):
            image = self.image.to_dict()

        name = self.name
        visible = self.visible

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if category_seo_tags is not UNSET:
            field_dict["categorySeoTags"] = category_seo_tags
        if description is not UNSET:
            field_dict["description"] = description
        if id is not UNSET:
            field_dict["id"] = id
        if image is not UNSET:
            field_dict["image"] = image
        if name is not UNSET:
            field_dict["name"] = name
        if visible is not UNSET:
            field_dict["visible"] = visible

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        category_seo_tags = []
        _category_seo_tags = d.pop("categorySeoTags", UNSET)
        for category_seo_tags_item_data in _category_seo_tags or []:
            category_seo_tags_item = CategorySeoTag.from_dict(category_seo_tags_item_data)

            category_seo_tags.append(category_seo_tags_item)

        description = d.pop("description", UNSET)

        id = d.pop("id", UNSET)

        _image = d.pop("image", UNSET)
        image: Union[Unset, Image]
        if isinstance(_image, Unset):
            image = UNSET
        else:
            image = Image.from_dict(_image)

        name = d.pop("name", UNSET)

        visible = d.pop("visible", UNSET)

        category = cls(
            category_seo_tags=category_seo_tags,
            description=description,
            id=id,
            image=image,
            name=name,
            visible=visible,
        )

        category.additional_properties = d
        return category

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.price_option_create_price_group_type import PriceOptionCreatePriceGroupType
from ..types import UNSET, Unset

T = TypeVar("T", bound="PriceOptionCreate")


@attr.s(auto_attribs=True)
class PriceOptionCreate:
    """A Price Option belongs to a product or products session. It holds the price details for a specific price
    type.Products can have one or many price options (I.e. Adult, Child, Family, etc.)

        Attributes:
            label (Union[Unset, str]): Label for this price (I.e. "Adult", "Child")
            max_quantity (Union[Unset, int]): Max booking quantity for the product price option. Can be specified, if the
                price option is fixed or a grouptype. For a successful booking of the product, the number of participants for
                this price option have to be lesser or equal than this value.
            min_quantity (Union[Unset, int]): Min booking quantity for the product price option. Can be specified, if the
                price option is fixed or a group type. For a successful booking of the product, the number of participants for
                this price option have to be greater or equal than this value.
            price (Union[Unset, float]): Price amount (I.e. "200.00")
            price_group_type (Union[Unset, PriceOptionCreatePriceGroupType]): If this price is for a group, is the price for
                the whole group (TOTAL) or per quantity (EACH)
            product_code (Union[Unset, str]): Product code to which the price options belongs to. Since Rezdy introduced
                shared availability option for products, the product sessions can contain price overrides for all of the
                products, which share the sessions. Therefore it is necessary to filer only the price options matching the
                chosen product code on the client side, when processing /availability service responses.
            seats_used (Union[Unset, int]): How many seats one quantity of this price willuse. Used for availability
                calculations. For example 1 quantity of "Family of 4" will use 4 seats.
    """

    label: Union[Unset, str] = UNSET
    max_quantity: Union[Unset, int] = UNSET
    min_quantity: Union[Unset, int] = UNSET
    price: Union[Unset, float] = UNSET
    price_group_type: Union[Unset, PriceOptionCreatePriceGroupType] = UNSET
    product_code: Union[Unset, str] = UNSET
    seats_used: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        label = self.label
        max_quantity = self.max_quantity
        min_quantity = self.min_quantity
        price = self.price
        price_group_type: Union[Unset, str] = UNSET
        if not isinstance(self.price_group_type, Unset):
            price_group_type = self.price_group_type.value

        product_code = self.product_code
        seats_used = self.seats_used

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if label is not UNSET:
            field_dict["label"] = label
        if max_quantity is not UNSET:
            field_dict["maxQuantity"] = max_quantity
        if min_quantity is not UNSET:
            field_dict["minQuantity"] = min_quantity
        if price is not UNSET:
            field_dict["price"] = price
        if price_group_type is not UNSET:
            field_dict["priceGroupType"] = price_group_type
        if product_code is not UNSET:
            field_dict["productCode"] = product_code
        if seats_used is not UNSET:
            field_dict["seatsUsed"] = seats_used

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label = d.pop("label", UNSET)

        max_quantity = d.pop("maxQuantity", UNSET)

        min_quantity = d.pop("minQuantity", UNSET)

        price = d.pop("price", UNSET)

        _price_group_type = d.pop("priceGroupType", UNSET)
        price_group_type: Union[Unset, PriceOptionCreatePriceGroupType]
        if isinstance(_price_group_type, Unset):
            price_group_type = UNSET
        else:
            price_group_type = PriceOptionCreatePriceGroupType(_price_group_type)

        product_code = d.pop("productCode", UNSET)

        seats_used = d.pop("seatsUsed", UNSET)

        price_option_create = cls(
            label=label,
            max_quantity=max_quantity,
            min_quantity=min_quantity,
            price=price,
            price_group_type=price_group_type,
            product_code=product_code,
            seats_used=seats_used,
        )

        price_option_create.additional_properties = d
        return price_option_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class BookingCreateResellerSource(str, Enum):
    ONLINE = "ONLINE"
    INTERNAL = "INTERNAL"
    PARTNERS = "PARTNERS"
    COMMUNITY = "COMMUNITY"
    MARKETPLACE = "MARKETPLACE"
    MARKETPLACE_PREF_RATE = "MARKETPLACE_PREF_RATE"
    API = "API"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class BookingBarcodeType(str, Enum):
    TEXT = "TEXT"
    CODE_39 = "CODE_39"
    CODE_128 = "CODE_128"
    QR_CODE = "QR_CODE"
    EAN_8 = "EAN_8"
    EAN_13 = "EAN_13"
    ITF = "ITF"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.rezdy_connect_settings_external_api_format import RezdyConnectSettingsExternalApiFormat
from ..models.rezdy_connect_settings_external_booking_strategy import RezdyConnectSettingsExternalBookingStrategy
from ..types import UNSET, Unset

T = TypeVar("T", bound="RezdyConnectSettings")


@attr.s(auto_attribs=True)
class RezdyConnectSettings:
    """Extended product model to support advanced product configuration, including RezdyConnect settings

    Attributes:
        product_code (str): Rezdy-generated unique Product code. Used by agents and for API calls
        external_api_format (Union[Unset, RezdyConnectSettingsExternalApiFormat]): Data format for an external inventory
            mode product. Use it to specifya payload data format for RezdyConnect calls, when creating a product.
        external_availability_api (Union[Unset, str]): External availability endpoint for an external inventory mode
            product. Use it to specify an availability endpoint for RezdyConnect calls, when creating a product.
        external_booking_api (Union[Unset, str]): External booking endpoint for an external inventory mode product. Use
            it to specify a booking endpoint for RezdyConnect calls, when creating a product.
        external_booking_strategy (Union[Unset, RezdyConnectSettingsExternalBookingStrategy]): Booking strategy for an
            external inventory mode product. Use it to specify a booking strategy for RezdyConnect calls, when creating a
            product.
        external_cancellation_api (Union[Unset, str]): External booking cancellation endpoint for an external inventory
            mode product. Use it to specify a cancellation endpoint for RezdyConnect calls, when creating a product.
        external_reservation_api (Union[Unset, str]): External reservation endpoint for an external inventory mode
            product. Use it to specify a reservation endpoint for RezdyConnect calls, when creating a product.
    """

    product_code: str
    external_api_format: Union[Unset, RezdyConnectSettingsExternalApiFormat] = UNSET
    external_availability_api: Union[Unset, str] = UNSET
    external_booking_api: Union[Unset, str] = UNSET
    external_booking_strategy: Union[Unset, RezdyConnectSettingsExternalBookingStrategy] = UNSET
    external_cancellation_api: Union[Unset, str] = UNSET
    external_reservation_api: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        product_code = self.product_code
        external_api_format: Union[Unset, str] = UNSET
        if not isinstance(self.external_api_format, Unset):
            external_api_format = self.external_api_format.value

        external_availability_api = self.external_availability_api
        external_booking_api = self.external_booking_api
        external_booking_strategy: Union[Unset, str] = UNSET
        if not isinstance(self.external_booking_strategy, Unset):
            external_booking_strategy = self.external_booking_strategy.value

        external_cancellation_api = self.external_cancellation_api
        external_reservation_api = self.external_reservation_api

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "productCode": product_code,
            }
        )
        if external_api_format is not UNSET:
            field_dict["externalApiFormat"] = external_api_format
        if external_availability_api is not UNSET:
            field_dict["externalAvailabilityApi"] = external_availability_api
        if external_booking_api is not UNSET:
            field_dict["externalBookingApi"] = external_booking_api
        if external_booking_strategy is not UNSET:
            field_dict["externalBookingStrategy"] = external_booking_strategy
        if external_cancellation_api is not UNSET:
            field_dict["externalCancellationApi"] = external_cancellation_api
        if external_reservation_api is not UNSET:
            field_dict["externalReservationApi"] = external_reservation_api

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        product_code = d.pop("productCode")

        _external_api_format = d.pop("externalApiFormat", UNSET)
        external_api_format: Union[Unset, RezdyConnectSettingsExternalApiFormat]
        if isinstance(_external_api_format, Unset):
            external_api_format = UNSET
        else:
            external_api_format = RezdyConnectSettingsExternalApiFormat(_external_api_format)

        external_availability_api = d.pop("externalAvailabilityApi", UNSET)

        external_booking_api = d.pop("externalBookingApi", UNSET)

        _external_booking_strategy = d.pop("externalBookingStrategy", UNSET)
        external_booking_strategy: Union[Unset, RezdyConnectSettingsExternalBookingStrategy]
        if isinstance(_external_booking_strategy, Unset):
            external_booking_strategy = UNSET
        else:
            external_booking_strategy = RezdyConnectSettingsExternalBookingStrategy(_external_booking_strategy)

        external_cancellation_api = d.pop("externalCancellationApi", UNSET)

        external_reservation_api = d.pop("externalReservationApi", UNSET)

        rezdy_connect_settings = cls(
            product_code=product_code,
            external_api_format=external_api_format,
            external_availability_api=external_availability_api,
            external_booking_api=external_booking_api,
            external_booking_strategy=external_booking_strategy,
            external_cancellation_api=external_cancellation_api,
            external_reservation_api=external_reservation_api,
        )

        rezdy_connect_settings.additional_properties = d
        return rezdy_connect_settings

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.booking_update import BookingUpdate
from ...models.response_booking import ResponseBooking
from ...types import Response


def _get_kwargs(
    order_number: str,
    *,
    client: Client,
    json_body: BookingUpdate,
) -> Dict[str, Any]:
    url = "{}/bookings/{orderNumber}".format(client.base_url, orderNumber=order_number)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[ResponseBooking]:
    if response.status_code == 200:
        response_200 = ResponseBooking.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[ResponseBooking]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    order_number: str,
    *,
    client: Client,
    json_body: BookingUpdate,
) -> Response[ResponseBooking]:
    """Update booking

     When implementing booking update take into consideration below:

    - only certain fields can currently be updated using API
    - only manual payment bookings can be updated
    - update booking in API triggers webhooks and e-mail notifications in the same way as the order
    update through UI
    - good practice is to retrieve the full booking object either from create or get booking response,
    update the necessary fields and pass it whole back to the booking update service. In the future we
    might support updates of additional fields e.g. add and delete of participants. If you don't send
    the participants array in the request, we will recognize it as the participants deletion and remove
    the participant from the existing order
    - order of the items in arrays have to be preserved for the following fields \"items\",
    \"participants\", since no ids are exposed in the API and thus are matched based on the position in
    the array again the existing booking object
    - agent can update supplier orders only if the supplier allow them to *edit orders* when sharing
    their products

    The service method does not support a partial update, **full booking object, as it was retrieved
    from the booking create or search services**, has to be send back to the request payload.
     Otherwise, the properties or relations which <i>are currently supported (see below)</i> and they
    are not sent, will be deleted.
     **Order of the items in arrays have to be preserved for the following fields 'items',
    'participants'.**

    **Currently supported** fields are:

    - Booking.customer - all customer data can be updated
    - Booking.field - all 'per booking' booking fields values
    - Booking.item.participant.field - all 'per participant' booking fields values
    - Booking.resellerComments - both the booking agent and the supplier can update the booking
    resellerComments
    - Booking.resellerReference - both the booking agent and the supplier can update the booking
    resellerReference
    - Booking.items.pickupLocation.locationName - both the booking agent and the supplier can update the
    booking pickup location

    For the sample requests provided in the right panel, consider the booking object below being
    retrieved from a POST order or GET order methods:

    ```
    {
        \"requestStatus\": {
                \"success\": true,
                \"version\": \"v1\"
        },
        \"booking\": {
                \"orderNumber\": \"RSKCJ1K\",
                \"status\": \"CONFIRMED\",
                \"supplierId\": 61,
                \"supplierName\": \"SUPPLIER_PREMIUM_AU\",
                \"customer\": {
                        \"id\": 2,
                        \"firstName\": \"Dusan\",
                        \"lastName\": \"Zahoransky\",
                        \"name\": \"Dusan Zahoransky\",
                        \"email\": \"sample@test.com\"
                },
                \"items\": [
                        {
                                \"productName\": \"activity i session seats pp adult 100f\",
                                \"productCode\": \"P123456\",
                                \"startTime\": \"2017-01-19T09:00:00Z\",
                                \"endTime\": \"2017-01-19T11:00:00Z\",
                                \"startTimeLocal\": \"2017-01-19 20:00:00\",
                                \"endTimeLocal\": \"2017-01-19 22:00:00\",
                                \"quantities\": [
                                        {
                                                \"optionLabel\": \"Adult\",
                                                \"optionPrice\": 100,
                                                \"value\": 1
                                        }
                                ],
                                \"totalQuantity\": 1,
                                \"amount\": 100,
                                \"extras\": [
                                ],
                                \"participants\": [
                                        {
                                                \"fields\": [
                                                        {
                                                                \"label\": \"First Name\",
                                                                \"value\": \"Janko\",
                                                                \"requiredPerParticipant\": false,
                                                                \"requiredPerBooking\": false,
                                                                \"visiblePerParticipant\": false,
                                                                \"visiblePerBooking\": false
                                                        },
                                                        {
                                                                \"label\": \"Last Name\",
                                                                \"value\": \"Hrasko\",
                                                                \"requiredPerParticipant\": false,
                                                                \"requiredPerBooking\": false,
                                                                \"visiblePerParticipant\": false,
                                                                \"visiblePerBooking\": false
                                                        }
                                                ]
                                        }
                                ],
                                \"subtotal\": 100,
                                \"vouchers\": [
                                ]
                        }
                ],
                \"totalAmount\": 100,
                \"totalCurrency\": \"AUD\",
                \"totalPaid\": 0,
                \"totalDue\": 100,
                \"dateCreated\": \"2017-01-19T03:36:18.462Z\",
                \"dateConfirmed\": \"2017-01-19T03:36:18.462Z\",
                \"payments\": [
                ],
                \"fields\": [
                        {
                                \"label\": \"Special Requirements\",
                                \"value\": \"No meat meal option\",
                                \"requiredPerParticipant\": false,
                                \"requiredPerBooking\": false,
                                \"visiblePerParticipant\": false,
                                \"visiblePerBooking\": false
                        }
                ],
                \"source\": \"API\",
                \"vouchers\": [
                ]
        }
    }
    ```

    Args:
        order_number (str):
        json_body (BookingUpdate): Booking update object used to update a booking in Rezdy's
            system.

    Returns:
        Response[ResponseBooking]
    """

    kwargs = _get_kwargs(
        order_number=order_number,
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    order_number: str,
    *,
    client: Client,
    json_body: BookingUpdate,
) -> Optional[ResponseBooking]:
    """Update booking

     When implementing booking update take into consideration below:

    - only certain fields can currently be updated using API
    - only manual payment bookings can be updated
    - update booking in API triggers webhooks and e-mail notifications in the same way as the order
    update through UI
    - good practice is to retrieve the full booking object either from create or get booking response,
    update the necessary fields and pass it whole back to the booking update service. In the future we
    might support updates of additional fields e.g. add and delete of participants. If you don't send
    the participants array in the request, we will recognize it as the participants deletion and remove
    the participant from the existing order
    - order of the items in arrays have to be preserved for the following fields \"items\",
    \"participants\", since no ids are exposed in the API and thus are matched based on the position in
    the array again the existing booking object
    - agent can update supplier orders only if the supplier allow them to *edit orders* when sharing
    their products

    The service method does not support a partial update, **full booking object, as it was retrieved
    from the booking create or search services**, has to be send back to the request payload.
     Otherwise, the properties or relations which <i>are currently supported (see below)</i> and they
    are not sent, will be deleted.
     **Order of the items in arrays have to be preserved for the following fields 'items',
    'participants'.**

    **Currently supported** fields are:

    - Booking.customer - all customer data can be updated
    - Booking.field - all 'per booking' booking fields values
    - Booking.item.participant.field - all 'per participant' booking fields values
    - Booking.resellerComments - both the booking agent and the supplier can update the booking
    resellerComments
    - Booking.resellerReference - both the booking agent and the supplier can update the booking
    resellerReference
    - Booking.items.pickupLocation.locationName - both the booking agent and the supplier can update the
    booking pickup location

    For the sample requests provided in the right panel, consider the booking object below being
    retrieved from a POST order or GET order methods:

    ```
    {
        \"requestStatus\": {
                \"success\": true,
                \"version\": \"v1\"
        },
        \"booking\": {
                \"orderNumber\": \"RSKCJ1K\",
                \"status\": \"CONFIRMED\",
                \"supplierId\": 61,
                \"supplierName\": \"SUPPLIER_PREMIUM_AU\",
                \"customer\": {
                        \"id\": 2,
                        \"firstName\": \"Dusan\",
                        \"lastName\": \"Zahoransky\",
                        \"name\": \"Dusan Zahoransky\",
                        \"email\": \"sample@test.com\"
                },
                \"items\": [
                        {
                                \"productName\": \"activity i session seats pp adult 100f\",
                                \"productCode\": \"P123456\",
                                \"startTime\": \"2017-01-19T09:00:00Z\",
                                \"endTime\": \"2017-01-19T11:00:00Z\",
                                \"startTimeLocal\": \"2017-01-19 20:00:00\",
                                \"endTimeLocal\": \"2017-01-19 22:00:00\",
                                \"quantities\": [
                                        {
                                                \"optionLabel\": \"Adult\",
                                                \"optionPrice\": 100,
                                                \"value\": 1
                                        }
                                ],
                                \"totalQuantity\": 1,
                                \"amount\": 100,
                                \"extras\": [
                                ],
                                \"participants\": [
                                        {
                                                \"fields\": [
                                                        {
                                                                \"label\": \"First Name\",
                                                                \"value\": \"Janko\",
                                                                \"requiredPerParticipant\": false,
                                                                \"requiredPerBooking\": false,
                                                                \"visiblePerParticipant\": false,
                                                                \"visiblePerBooking\": false
                                                        },
                                                        {
                                                                \"label\": \"Last Name\",
                                                                \"value\": \"Hrasko\",
                                                                \"requiredPerParticipant\": false,
                                                                \"requiredPerBooking\": false,
                                                                \"visiblePerParticipant\": false,
                                                                \"visiblePerBooking\": false
                                                        }
                                                ]
                                        }
                                ],
                                \"subtotal\": 100,
                                \"vouchers\": [
                                ]
                        }
                ],
                \"totalAmount\": 100,
                \"totalCurrency\": \"AUD\",
                \"totalPaid\": 0,
                \"totalDue\": 100,
                \"dateCreated\": \"2017-01-19T03:36:18.462Z\",
                \"dateConfirmed\": \"2017-01-19T03:36:18.462Z\",
                \"payments\": [
                ],
                \"fields\": [
                        {
                                \"label\": \"Special Requirements\",
                                \"value\": \"No meat meal option\",
                                \"requiredPerParticipant\": false,
                                \"requiredPerBooking\": false,
                                \"visiblePerParticipant\": false,
                                \"visiblePerBooking\": false
                        }
                ],
                \"source\": \"API\",
                \"vouchers\": [
                ]
        }
    }
    ```

    Args:
        order_number (str):
        json_body (BookingUpdate): Booking update object used to update a booking in Rezdy's
            system.

    Returns:
        Response[ResponseBooking]
    """

    return sync_detailed(
        order_number=order_number,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    order_number: str,
    *,
    client: Client,
    json_body: BookingUpdate,
) -> Response[ResponseBooking]:
    """Update booking

     When implementing booking update take into consideration below:

    - only certain fields can currently be updated using API
    - only manual payment bookings can be updated
    - update booking in API triggers webhooks and e-mail notifications in the same way as the order
    update through UI
    - good practice is to retrieve the full booking object either from create or get booking response,
    update the necessary fields and pass it whole back to the booking update service. In the future we
    might support updates of additional fields e.g. add and delete of participants. If you don't send
    the participants array in the request, we will recognize it as the participants deletion and remove
    the participant from the existing order
    - order of the items in arrays have to be preserved for the following fields \"items\",
    \"participants\", since no ids are exposed in the API and thus are matched based on the position in
    the array again the existing booking object
    - agent can update supplier orders only if the supplier allow them to *edit orders* when sharing
    their products

    The service method does not support a partial update, **full booking object, as it was retrieved
    from the booking create or search services**, has to be send back to the request payload.
     Otherwise, the properties or relations which <i>are currently supported (see below)</i> and they
    are not sent, will be deleted.
     **Order of the items in arrays have to be preserved for the following fields 'items',
    'participants'.**

    **Currently supported** fields are:

    - Booking.customer - all customer data can be updated
    - Booking.field - all 'per booking' booking fields values
    - Booking.item.participant.field - all 'per participant' booking fields values
    - Booking.resellerComments - both the booking agent and the supplier can update the booking
    resellerComments
    - Booking.resellerReference - both the booking agent and the supplier can update the booking
    resellerReference
    - Booking.items.pickupLocation.locationName - both the booking agent and the supplier can update the
    booking pickup location

    For the sample requests provided in the right panel, consider the booking object below being
    retrieved from a POST order or GET order methods:

    ```
    {
        \"requestStatus\": {
                \"success\": true,
                \"version\": \"v1\"
        },
        \"booking\": {
                \"orderNumber\": \"RSKCJ1K\",
                \"status\": \"CONFIRMED\",
                \"supplierId\": 61,
                \"supplierName\": \"SUPPLIER_PREMIUM_AU\",
                \"customer\": {
                        \"id\": 2,
                        \"firstName\": \"Dusan\",
                        \"lastName\": \"Zahoransky\",
                        \"name\": \"Dusan Zahoransky\",
                        \"email\": \"sample@test.com\"
                },
                \"items\": [
                        {
                                \"productName\": \"activity i session seats pp adult 100f\",
                                \"productCode\": \"P123456\",
                                \"startTime\": \"2017-01-19T09:00:00Z\",
                                \"endTime\": \"2017-01-19T11:00:00Z\",
                                \"startTimeLocal\": \"2017-01-19 20:00:00\",
                                \"endTimeLocal\": \"2017-01-19 22:00:00\",
                                \"quantities\": [
                                        {
                                                \"optionLabel\": \"Adult\",
                                                \"optionPrice\": 100,
                                                \"value\": 1
                                        }
                                ],
                                \"totalQuantity\": 1,
                                \"amount\": 100,
                                \"extras\": [
                                ],
                                \"participants\": [
                                        {
                                                \"fields\": [
                                                        {
                                                                \"label\": \"First Name\",
                                                                \"value\": \"Janko\",
                                                                \"requiredPerParticipant\": false,
                                                                \"requiredPerBooking\": false,
                                                                \"visiblePerParticipant\": false,
                                                                \"visiblePerBooking\": false
                                                        },
                                                        {
                                                                \"label\": \"Last Name\",
                                                                \"value\": \"Hrasko\",
                                                                \"requiredPerParticipant\": false,
                                                                \"requiredPerBooking\": false,
                                                                \"visiblePerParticipant\": false,
                                                                \"visiblePerBooking\": false
                                                        }
                                                ]
                                        }
                                ],
                                \"subtotal\": 100,
                                \"vouchers\": [
                                ]
                        }
                ],
                \"totalAmount\": 100,
                \"totalCurrency\": \"AUD\",
                \"totalPaid\": 0,
                \"totalDue\": 100,
                \"dateCreated\": \"2017-01-19T03:36:18.462Z\",
                \"dateConfirmed\": \"2017-01-19T03:36:18.462Z\",
                \"payments\": [
                ],
                \"fields\": [
                        {
                                \"label\": \"Special Requirements\",
                                \"value\": \"No meat meal option\",
                                \"requiredPerParticipant\": false,
                                \"requiredPerBooking\": false,
                                \"visiblePerParticipant\": false,
                                \"visiblePerBooking\": false
                        }
                ],
                \"source\": \"API\",
                \"vouchers\": [
                ]
        }
    }
    ```

    Args:
        order_number (str):
        json_body (BookingUpdate): Booking update object used to update a booking in Rezdy's
            system.

    Returns:
        Response[ResponseBooking]
    """

    kwargs = _get_kwargs(
        order_number=order_number,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    order_number: str,
    *,
    client: Client,
    json_body: BookingUpdate,
) -> Optional[ResponseBooking]:
    """Update booking

     When implementing booking update take into consideration below:

    - only certain fields can currently be updated using API
    - only manual payment bookings can be updated
    - update booking in API triggers webhooks and e-mail notifications in the same way as the order
    update through UI
    - good practice is to retrieve the full booking object either from create or get booking response,
    update the necessary fields and pass it whole back to the booking update service. In the future we
    might support updates of additional fields e.g. add and delete of participants. If you don't send
    the participants array in the request, we will recognize it as the participants deletion and remove
    the participant from the existing order
    - order of the items in arrays have to be preserved for the following fields \"items\",
    \"participants\", since no ids are exposed in the API and thus are matched based on the position in
    the array again the existing booking object
    - agent can update supplier orders only if the supplier allow them to *edit orders* when sharing
    their products

    The service method does not support a partial update, **full booking object, as it was retrieved
    from the booking create or search services**, has to be send back to the request payload.
     Otherwise, the properties or relations which <i>are currently supported (see below)</i> and they
    are not sent, will be deleted.
     **Order of the items in arrays have to be preserved for the following fields 'items',
    'participants'.**

    **Currently supported** fields are:

    - Booking.customer - all customer data can be updated
    - Booking.field - all 'per booking' booking fields values
    - Booking.item.participant.field - all 'per participant' booking fields values
    - Booking.resellerComments - both the booking agent and the supplier can update the booking
    resellerComments
    - Booking.resellerReference - both the booking agent and the supplier can update the booking
    resellerReference
    - Booking.items.pickupLocation.locationName - both the booking agent and the supplier can update the
    booking pickup location

    For the sample requests provided in the right panel, consider the booking object below being
    retrieved from a POST order or GET order methods:

    ```
    {
        \"requestStatus\": {
                \"success\": true,
                \"version\": \"v1\"
        },
        \"booking\": {
                \"orderNumber\": \"RSKCJ1K\",
                \"status\": \"CONFIRMED\",
                \"supplierId\": 61,
                \"supplierName\": \"SUPPLIER_PREMIUM_AU\",
                \"customer\": {
                        \"id\": 2,
                        \"firstName\": \"Dusan\",
                        \"lastName\": \"Zahoransky\",
                        \"name\": \"Dusan Zahoransky\",
                        \"email\": \"sample@test.com\"
                },
                \"items\": [
                        {
                                \"productName\": \"activity i session seats pp adult 100f\",
                                \"productCode\": \"P123456\",
                                \"startTime\": \"2017-01-19T09:00:00Z\",
                                \"endTime\": \"2017-01-19T11:00:00Z\",
                                \"startTimeLocal\": \"2017-01-19 20:00:00\",
                                \"endTimeLocal\": \"2017-01-19 22:00:00\",
                                \"quantities\": [
                                        {
                                                \"optionLabel\": \"Adult\",
                                                \"optionPrice\": 100,
                                                \"value\": 1
                                        }
                                ],
                                \"totalQuantity\": 1,
                                \"amount\": 100,
                                \"extras\": [
                                ],
                                \"participants\": [
                                        {
                                                \"fields\": [
                                                        {
                                                                \"label\": \"First Name\",
                                                                \"value\": \"Janko\",
                                                                \"requiredPerParticipant\": false,
                                                                \"requiredPerBooking\": false,
                                                                \"visiblePerParticipant\": false,
                                                                \"visiblePerBooking\": false
                                                        },
                                                        {
                                                                \"label\": \"Last Name\",
                                                                \"value\": \"Hrasko\",
                                                                \"requiredPerParticipant\": false,
                                                                \"requiredPerBooking\": false,
                                                                \"visiblePerParticipant\": false,
                                                                \"visiblePerBooking\": false
                                                        }
                                                ]
                                        }
                                ],
                                \"subtotal\": 100,
                                \"vouchers\": [
                                ]
                        }
                ],
                \"totalAmount\": 100,
                \"totalCurrency\": \"AUD\",
                \"totalPaid\": 0,
                \"totalDue\": 100,
                \"dateCreated\": \"2017-01-19T03:36:18.462Z\",
                \"dateConfirmed\": \"2017-01-19T03:36:18.462Z\",
                \"payments\": [
                ],
                \"fields\": [
                        {
                                \"label\": \"Special Requirements\",
                                \"value\": \"No meat meal option\",
                                \"requiredPerParticipant\": false,
                                \"requiredPerBooking\": false,
                                \"visiblePerParticipant\": false,
                                \"visiblePerBooking\": false
                        }
                ],
                \"source\": \"API\",
                \"vouchers\": [
                ]
        }
    }
    ```

    Args:
        order_number (str):
        json_body (BookingUpdate): Booking update object used to update a booking in Rezdy's
            system.

    Returns:
        Response[ResponseBooking]
    """

    return (
        await asyncio_detailed(
            order_number=order_number,
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.pickup_location import PickupLocation
from ..types import UNSET, Unset

T = TypeVar("T", bound="PickupList")


@attr.s(auto_attribs=True)
class PickupList:
    """PickupList object. Contains a list of pickup locations.

    Attributes:
        additional_notes (Union[Unset, str]): Global additional instructions for this pick up list
        id (Union[Unset, int]): ID of this pickup
        name (Union[Unset, str]): Name of the pickup location list
        other_location_instructions (Union[Unset, str]): Instructions for other locations that are not available in the
            pickupLocations list. E.g. For customer pick up location requests, a sample instruction for this field would be:
            'We will contact you to confirm your pickup location'
        pickup_locations (Union[Unset, List[PickupLocation]]): List of all associated pickup locations for this list
    """

    additional_notes: Union[Unset, str] = UNSET
    id: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    other_location_instructions: Union[Unset, str] = UNSET
    pickup_locations: Union[Unset, List[PickupLocation]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        additional_notes = self.additional_notes
        id = self.id
        name = self.name
        other_location_instructions = self.other_location_instructions
        pickup_locations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.pickup_locations, Unset):
            pickup_locations = []
            for pickup_locations_item_data in self.pickup_locations:
                pickup_locations_item = pickup_locations_item_data.to_dict()

                pickup_locations.append(pickup_locations_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if additional_notes is not UNSET:
            field_dict["additionalNotes"] = additional_notes
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if other_location_instructions is not UNSET:
            field_dict["otherLocationInstructions"] = other_location_instructions
        if pickup_locations is not UNSET:
            field_dict["pickupLocations"] = pickup_locations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        additional_notes = d.pop("additionalNotes", UNSET)

        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        other_location_instructions = d.pop("otherLocationInstructions", UNSET)

        pickup_locations = []
        _pickup_locations = d.pop("pickupLocations", UNSET)
        for pickup_locations_item_data in _pickup_locations or []:
            pickup_locations_item = PickupLocation.from_dict(pickup_locations_item_data)

            pickup_locations.append(pickup_locations_item)

        pickup_list = cls(
            additional_notes=additional_notes,
            id=id,
            name=name,
            other_location_instructions=other_location_instructions,
            pickup_locations=pickup_locations,
        )

        pickup_list.additional_properties = d
        return pickup_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.request_status import RequestStatus
from ..models.resource import Resource
from ..types import UNSET, Unset

T = TypeVar("T", bound="ResponseResource")


@attr.s(auto_attribs=True)
class ResponseResource:
    """
    Attributes:
        request_status (RequestStatus):
        resource (Union[Unset, Resource]): Supplier resource - e.g. raft, bus, tour guide, venue which has a limited
            capacity. The resources can be shared between different supplier's products. If the resource does not have any
            spare availability, the booking of any of the product sessions, where the resource is used will not be possible.
    """

    request_status: RequestStatus
    resource: Union[Unset, Resource] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_status = self.request_status.to_dict()

        resource: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.resource, Unset):
            resource = self.resource.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestStatus": request_status,
            }
        )
        if resource is not UNSET:
            field_dict["resource"] = resource

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        request_status = RequestStatus.from_dict(d.pop("requestStatus"))

        _resource = d.pop("resource", UNSET)
        resource: Union[Unset, Resource]
        if isinstance(_resource, Unset):
            resource = UNSET
        else:
            resource = Resource.from_dict(_resource)

        response_resource = cls(
            request_status=request_status,
            resource=resource,
        )

        response_resource.additional_properties = d
        return response_resource

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.response_booking_list import ResponseBookingList
from ...models.search_bookings_order_status import SearchBookingsOrderStatus
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    order_status: Union[Unset, None, SearchBookingsOrderStatus] = UNSET,
    search: Union[Unset, None, str] = UNSET,
    product_code: Union[Unset, None, List[str]] = UNSET,
    min_tour_start_time: Union[Unset, None, str] = UNSET,
    max_tour_start_time: Union[Unset, None, str] = UNSET,
    updated_since: Union[Unset, None, str] = UNSET,
    min_date_created: Union[Unset, None, str] = UNSET,
    max_date_created: Union[Unset, None, str] = UNSET,
    reseller_reference: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Dict[str, Any]:
    url = "{}/bookings".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_order_status: Union[Unset, None, str] = UNSET
    if not isinstance(order_status, Unset):
        json_order_status = order_status.value if order_status else None

    params["orderStatus"] = json_order_status

    params["search"] = search

    json_product_code: Union[Unset, None, List[str]] = UNSET
    if not isinstance(product_code, Unset):
        if product_code is None:
            json_product_code = None
        else:
            json_product_code = product_code

    params["productCode"] = json_product_code

    params["minTourStartTime"] = min_tour_start_time

    params["maxTourStartTime"] = max_tour_start_time

    params["updatedSince"] = updated_since

    params["minDateCreated"] = min_date_created

    params["maxDateCreated"] = max_date_created

    params["resellerReference"] = reseller_reference

    params["limit"] = limit

    params["offset"] = offset

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[ResponseBookingList]:
    if response.status_code == 200:
        response_200 = ResponseBookingList.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[ResponseBookingList]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    order_status: Union[Unset, None, SearchBookingsOrderStatus] = UNSET,
    search: Union[Unset, None, str] = UNSET,
    product_code: Union[Unset, None, List[str]] = UNSET,
    min_tour_start_time: Union[Unset, None, str] = UNSET,
    max_tour_start_time: Union[Unset, None, str] = UNSET,
    updated_since: Union[Unset, None, str] = UNSET,
    min_date_created: Union[Unset, None, str] = UNSET,
    max_date_created: Union[Unset, None, str] = UNSET,
    reseller_reference: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Response[ResponseBookingList]:
    """Search bookings

     Search bookings in your account

    Args:
        order_status (Union[Unset, None, SearchBookingsOrderStatus]):
        search (Union[Unset, None, str]):
        product_code (Union[Unset, None, List[str]]):
        min_tour_start_time (Union[Unset, None, str]):
        max_tour_start_time (Union[Unset, None, str]):
        updated_since (Union[Unset, None, str]):
        min_date_created (Union[Unset, None, str]):
        max_date_created (Union[Unset, None, str]):
        reseller_reference (Union[Unset, None, str]):
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[ResponseBookingList]
    """

    kwargs = _get_kwargs(
        client=client,
        order_status=order_status,
        search=search,
        product_code=product_code,
        min_tour_start_time=min_tour_start_time,
        max_tour_start_time=max_tour_start_time,
        updated_since=updated_since,
        min_date_created=min_date_created,
        max_date_created=max_date_created,
        reseller_reference=reseller_reference,
        limit=limit,
        offset=offset,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    order_status: Union[Unset, None, SearchBookingsOrderStatus] = UNSET,
    search: Union[Unset, None, str] = UNSET,
    product_code: Union[Unset, None, List[str]] = UNSET,
    min_tour_start_time: Union[Unset, None, str] = UNSET,
    max_tour_start_time: Union[Unset, None, str] = UNSET,
    updated_since: Union[Unset, None, str] = UNSET,
    min_date_created: Union[Unset, None, str] = UNSET,
    max_date_created: Union[Unset, None, str] = UNSET,
    reseller_reference: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Optional[ResponseBookingList]:
    """Search bookings

     Search bookings in your account

    Args:
        order_status (Union[Unset, None, SearchBookingsOrderStatus]):
        search (Union[Unset, None, str]):
        product_code (Union[Unset, None, List[str]]):
        min_tour_start_time (Union[Unset, None, str]):
        max_tour_start_time (Union[Unset, None, str]):
        updated_since (Union[Unset, None, str]):
        min_date_created (Union[Unset, None, str]):
        max_date_created (Union[Unset, None, str]):
        reseller_reference (Union[Unset, None, str]):
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[ResponseBookingList]
    """

    return sync_detailed(
        client=client,
        order_status=order_status,
        search=search,
        product_code=product_code,
        min_tour_start_time=min_tour_start_time,
        max_tour_start_time=max_tour_start_time,
        updated_since=updated_since,
        min_date_created=min_date_created,
        max_date_created=max_date_created,
        reseller_reference=reseller_reference,
        limit=limit,
        offset=offset,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    order_status: Union[Unset, None, SearchBookingsOrderStatus] = UNSET,
    search: Union[Unset, None, str] = UNSET,
    product_code: Union[Unset, None, List[str]] = UNSET,
    min_tour_start_time: Union[Unset, None, str] = UNSET,
    max_tour_start_time: Union[Unset, None, str] = UNSET,
    updated_since: Union[Unset, None, str] = UNSET,
    min_date_created: Union[Unset, None, str] = UNSET,
    max_date_created: Union[Unset, None, str] = UNSET,
    reseller_reference: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Response[ResponseBookingList]:
    """Search bookings

     Search bookings in your account

    Args:
        order_status (Union[Unset, None, SearchBookingsOrderStatus]):
        search (Union[Unset, None, str]):
        product_code (Union[Unset, None, List[str]]):
        min_tour_start_time (Union[Unset, None, str]):
        max_tour_start_time (Union[Unset, None, str]):
        updated_since (Union[Unset, None, str]):
        min_date_created (Union[Unset, None, str]):
        max_date_created (Union[Unset, None, str]):
        reseller_reference (Union[Unset, None, str]):
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[ResponseBookingList]
    """

    kwargs = _get_kwargs(
        client=client,
        order_status=order_status,
        search=search,
        product_code=product_code,
        min_tour_start_time=min_tour_start_time,
        max_tour_start_time=max_tour_start_time,
        updated_since=updated_since,
        min_date_created=min_date_created,
        max_date_created=max_date_created,
        reseller_reference=reseller_reference,
        limit=limit,
        offset=offset,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    order_status: Union[Unset, None, SearchBookingsOrderStatus] = UNSET,
    search: Union[Unset, None, str] = UNSET,
    product_code: Union[Unset, None, List[str]] = UNSET,
    min_tour_start_time: Union[Unset, None, str] = UNSET,
    max_tour_start_time: Union[Unset, None, str] = UNSET,
    updated_since: Union[Unset, None, str] = UNSET,
    min_date_created: Union[Unset, None, str] = UNSET,
    max_date_created: Union[Unset, None, str] = UNSET,
    reseller_reference: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Optional[ResponseBookingList]:
    """Search bookings

     Search bookings in your account

    Args:
        order_status (Union[Unset, None, SearchBookingsOrderStatus]):
        search (Union[Unset, None, str]):
        product_code (Union[Unset, None, List[str]]):
        min_tour_start_time (Union[Unset, None, str]):
        max_tour_start_time (Union[Unset, None, str]):
        updated_since (Union[Unset, None, str]):
        min_date_created (Union[Unset, None, str]):
        max_date_created (Union[Unset, None, str]):
        reseller_reference (Union[Unset, None, str]):
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[ResponseBookingList]
    """

    return (
        await asyncio_detailed(
            client=client,
            order_status=order_status,
            search=search,
            product_code=product_code,
            min_tour_start_time=min_tour_start_time,
            max_tour_start_time=max_tour_start_time,
            updated_since=updated_since,
            min_date_created=min_date_created,
            max_date_created=max_date_created,
            reseller_reference=reseller_reference,
            limit=limit,
            offset=offset,
        )
    ).parsed



from enum import Enum


class ProductAgentPaymentType(str, Enum):
    PAYOUTS = "PAYOUTS"
    FULL_AGENT = "FULL_AGENT"
    DOWNPAYMENT = "DOWNPAYMENT"
    FULL_SUPPLIER = "FULL_SUPPLIER"
    NONE = "NONE"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.extra import Extra
from ..models.participant import Participant
from ..models.pickup_location import PickupLocation
from ..models.quantity import Quantity
from ..models.voucher import Voucher
from ..types import UNSET, Unset

T = TypeVar("T", bound="BookingItem")


@attr.s(auto_attribs=True)
class BookingItem:
    """A BookingItem is a unique product/startTime combination in a booking

    Attributes:
        amount (Union[Unset, float]): Amount charged for this BookingItem. This is automatically generated based on
            quantities, but you can override the amount by entering a value. If automated payment method is used for the
            booked product, the Amount of the booked item<br>has to be grater than Net Rate sum of the booked quantities and
            Rezdy processing fee.
        end_time (Union[Unset, datetime.datetime]): End time of the session for this BookingItem
        end_time_local (Union[Unset, str]): End time of the session for this BookingItem in supplier's local timezone.
        extras (Union[Unset, List[Extra]]): List of Extras booked with this product
        participants (Union[Unset, List[Participant]]): List of participants. Each participant object contains all the
            booking fields for a single participant.
        pickup_location (Union[Unset, PickupLocation]): PickupLocation object. Holds information about the a pickup
            location from the pickup list configured for the product.
        product_code (Union[Unset, str]): Unique Rezdy code of the product in this BookingItem
        product_name (Union[Unset, str]): Name of the product in this BookingItem
        quantities (Union[Unset, List[Quantity]]): List of quantities booked by this item. Each Quantity must be linked
            to a Product price option via its label or ID.If the product only has one price option, only 'Quantity.value' is
            required.
        start_time (Union[Unset, datetime.datetime]): Start time of the session for this BookingItem
        start_time_local (Union[Unset, str]): Start time of the session for this BookingItem in supplier's local
            timezone.
        subtotal (Union[Unset, float]): Subtotal is the BookingItem.amount plus extras costs plus taxes and fees
        total_item_tax (Union[Unset, float]): Total tax applied to this booking item.
        total_quantity (Union[Unset, int]): Total quantity booked by all the Quantity. For example if the booking is for
            2 Adults and 1 Child, totalQuantity will be 3.
        transfer_from (Union[Unset, str]): From location. Only used by Shuttle and Transfer products
        transfer_return (Union[Unset, bool]): Is this a one-way (false) or a return trip (true)? Only used by Shuttle
            and Transfer products
        transfer_to (Union[Unset, str]): To location. Only used by Shuttle and Transfer products
        vouchers (Union[Unset, List[Voucher]]): List of vouchers created by this booking item - when the product is
            bought as a gift or is a gift card
    """

    amount: Union[Unset, float] = UNSET
    end_time: Union[Unset, datetime.datetime] = UNSET
    end_time_local: Union[Unset, str] = UNSET
    extras: Union[Unset, List[Extra]] = UNSET
    participants: Union[Unset, List[Participant]] = UNSET
    pickup_location: Union[Unset, PickupLocation] = UNSET
    product_code: Union[Unset, str] = UNSET
    product_name: Union[Unset, str] = UNSET
    quantities: Union[Unset, List[Quantity]] = UNSET
    start_time: Union[Unset, datetime.datetime] = UNSET
    start_time_local: Union[Unset, str] = UNSET
    subtotal: Union[Unset, float] = UNSET
    total_item_tax: Union[Unset, float] = UNSET
    total_quantity: Union[Unset, int] = UNSET
    transfer_from: Union[Unset, str] = UNSET
    transfer_return: Union[Unset, bool] = UNSET
    transfer_to: Union[Unset, str] = UNSET
    vouchers: Union[Unset, List[Voucher]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        amount = self.amount
        end_time: Union[Unset, str] = UNSET
        if not isinstance(self.end_time, Unset):
            end_time = self.end_time.isoformat()

        end_time_local = self.end_time_local
        extras: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.extras, Unset):
            extras = []
            for extras_item_data in self.extras:
                extras_item = extras_item_data.to_dict()

                extras.append(extras_item)

        participants: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.participants, Unset):
            participants = []
            for participants_item_data in self.participants:
                participants_item = participants_item_data.to_dict()

                participants.append(participants_item)

        pickup_location: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pickup_location, Unset):
            pickup_location = self.pickup_location.to_dict()

        product_code = self.product_code
        product_name = self.product_name
        quantities: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.quantities, Unset):
            quantities = []
            for quantities_item_data in self.quantities:
                quantities_item = quantities_item_data.to_dict()

                quantities.append(quantities_item)

        start_time: Union[Unset, str] = UNSET
        if not isinstance(self.start_time, Unset):
            start_time = self.start_time.isoformat()

        start_time_local = self.start_time_local
        subtotal = self.subtotal
        total_item_tax = self.total_item_tax
        total_quantity = self.total_quantity
        transfer_from = self.transfer_from
        transfer_return = self.transfer_return
        transfer_to = self.transfer_to
        vouchers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.vouchers, Unset):
            vouchers = []
            for vouchers_item_data in self.vouchers:
                vouchers_item = vouchers_item_data.to_dict()

                vouchers.append(vouchers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if amount is not UNSET:
            field_dict["amount"] = amount
        if end_time is not UNSET:
            field_dict["endTime"] = end_time
        if end_time_local is not UNSET:
            field_dict["endTimeLocal"] = end_time_local
        if extras is not UNSET:
            field_dict["extras"] = extras
        if participants is not UNSET:
            field_dict["participants"] = participants
        if pickup_location is not UNSET:
            field_dict["pickupLocation"] = pickup_location
        if product_code is not UNSET:
            field_dict["productCode"] = product_code
        if product_name is not UNSET:
            field_dict["productName"] = product_name
        if quantities is not UNSET:
            field_dict["quantities"] = quantities
        if start_time is not UNSET:
            field_dict["startTime"] = start_time
        if start_time_local is not UNSET:
            field_dict["startTimeLocal"] = start_time_local
        if subtotal is not UNSET:
            field_dict["subtotal"] = subtotal
        if total_item_tax is not UNSET:
            field_dict["totalItemTax"] = total_item_tax
        if total_quantity is not UNSET:
            field_dict["totalQuantity"] = total_quantity
        if transfer_from is not UNSET:
            field_dict["transferFrom"] = transfer_from
        if transfer_return is not UNSET:
            field_dict["transferReturn"] = transfer_return
        if transfer_to is not UNSET:
            field_dict["transferTo"] = transfer_to
        if vouchers is not UNSET:
            field_dict["vouchers"] = vouchers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        amount = d.pop("amount", UNSET)

        _end_time = d.pop("endTime", UNSET)
        end_time: Union[Unset, datetime.datetime]
        if isinstance(_end_time, Unset):
            end_time = UNSET
        else:
            end_time = isoparse(_end_time)

        end_time_local = d.pop("endTimeLocal", UNSET)

        extras = []
        _extras = d.pop("extras", UNSET)
        for extras_item_data in _extras or []:
            extras_item = Extra.from_dict(extras_item_data)

            extras.append(extras_item)

        participants = []
        _participants = d.pop("participants", UNSET)
        for participants_item_data in _participants or []:
            participants_item = Participant.from_dict(participants_item_data)

            participants.append(participants_item)

        _pickup_location = d.pop("pickupLocation", UNSET)
        pickup_location: Union[Unset, PickupLocation]
        if isinstance(_pickup_location, Unset):
            pickup_location = UNSET
        else:
            pickup_location = PickupLocation.from_dict(_pickup_location)

        product_code = d.pop("productCode", UNSET)

        product_name = d.pop("productName", UNSET)

        quantities = []
        _quantities = d.pop("quantities", UNSET)
        for quantities_item_data in _quantities or []:
            quantities_item = Quantity.from_dict(quantities_item_data)

            quantities.append(quantities_item)

        _start_time = d.pop("startTime", UNSET)
        start_time: Union[Unset, datetime.datetime]
        if isinstance(_start_time, Unset):
            start_time = UNSET
        else:
            start_time = isoparse(_start_time)

        start_time_local = d.pop("startTimeLocal", UNSET)

        subtotal = d.pop("subtotal", UNSET)

        total_item_tax = d.pop("totalItemTax", UNSET)

        total_quantity = d.pop("totalQuantity", UNSET)

        transfer_from = d.pop("transferFrom", UNSET)

        transfer_return = d.pop("transferReturn", UNSET)

        transfer_to = d.pop("transferTo", UNSET)

        vouchers = []
        _vouchers = d.pop("vouchers", UNSET)
        for vouchers_item_data in _vouchers or []:
            vouchers_item = Voucher.from_dict(vouchers_item_data)

            vouchers.append(vouchers_item)

        booking_item = cls(
            amount=amount,
            end_time=end_time,
            end_time_local=end_time_local,
            extras=extras,
            participants=participants,
            pickup_location=pickup_location,
            product_code=product_code,
            product_name=product_name,
            quantities=quantities,
            start_time=start_time,
            start_time_local=start_time_local,
            subtotal=subtotal,
            total_item_tax=total_item_tax,
            total_quantity=total_quantity,
            transfer_from=transfer_from,
            transfer_return=transfer_return,
            transfer_to=transfer_to,
            vouchers=vouchers,
        )

        booking_item.additional_properties = d
        return booking_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.response_resource_list import ResponseResourceList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Dict[str, Any]:
    url = "{}/resources".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["limit"] = limit

    params["offset"] = offset

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[ResponseResourceList]:
    if response.status_code == 200:
        response_200 = ResponseResourceList.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[ResponseResourceList]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Response[ResponseResourceList]:
    """List resources

     Retrieve all supplier resources. Pagination using limit and offset is applied to the result list.

    Args:
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[ResponseResourceList]
    """

    kwargs = _get_kwargs(
        client=client,
        limit=limit,
        offset=offset,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Optional[ResponseResourceList]:
    """List resources

     Retrieve all supplier resources. Pagination using limit and offset is applied to the result list.

    Args:
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[ResponseResourceList]
    """

    return sync_detailed(
        client=client,
        limit=limit,
        offset=offset,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Response[ResponseResourceList]:
    """List resources

     Retrieve all supplier resources. Pagination using limit and offset is applied to the result list.

    Args:
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[ResponseResourceList]
    """

    kwargs = _get_kwargs(
        client=client,
        limit=limit,
        offset=offset,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Optional[ResponseResourceList]:
    """List resources

     Retrieve all supplier resources. Pagination using limit and offset is applied to the result list.

    Args:
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[ResponseResourceList]
    """

    return (
        await asyncio_detailed(
            client=client,
            limit=limit,
            offset=offset,
        )
    ).parsed



from enum import Enum


class ProductBookingMode(str, Enum):
    NO_DATE = "NO_DATE"
    DATE_ENQUIRY = "DATE_ENQUIRY"
    INVENTORY = "INVENTORY"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.customer import Customer
from ..models.request_status import RequestStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ResponseCustomer")


@attr.s(auto_attribs=True)
class ResponseCustomer:
    """
    Attributes:
        request_status (RequestStatus):
        customer (Union[Unset, Customer]): The customer is the person making the booking, and most of the time paying
            for it.<br>It differs from Participants, who are the people attending a tour
    """

    request_status: RequestStatus
    customer: Union[Unset, Customer] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_status = self.request_status.to_dict()

        customer: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.customer, Unset):
            customer = self.customer.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestStatus": request_status,
            }
        )
        if customer is not UNSET:
            field_dict["customer"] = customer

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        request_status = RequestStatus.from_dict(d.pop("requestStatus"))

        _customer = d.pop("customer", UNSET)
        customer: Union[Unset, Customer]
        if isinstance(_customer, Unset):
            customer = UNSET
        else:
            customer = Customer.from_dict(_customer)

        response_customer = cls(
            request_status=request_status,
            customer=customer,
        )

        response_customer.additional_properties = d
        return response_customer

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
